/* 
 * Copyright 2000, 2001 The Regents of the University of California 
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 */

/*
 * $Id: crl_dnsstat.c,v 1.7.2.4 2001/11/21 01:33:41 kkeys Exp $
 */
/*
 * Keep statistics on DNS usage.
 */


static const char RCSid[]="$Id: crl_dnsstat.c,v 1.7.2.4 2001/11/21 01:33:41 kkeys Exp $";

/*#include "config.h"*/
#include <sys/param.h>
#include "caida_t.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>

#include "libcoral.h"
#include "hashtab/hashtab.h"

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <arpa/nameser.h>


/* Solaris crap */
#ifndef IP_OFFMASK
#define IP_OFFMASK 0x1FFF
#endif

#define DNS_PORT 53

struct {
    unsigned int ip_mask;
    unsigned int names;
    unsigned int hashstats;
    unsigned int nolabels;
    unsigned int nosrc;
    unsigned int nodst;
    unsigned int noproto;
    unsigned int human;
    unsigned int notypes;
    unsigned int no_rd;
    unsigned int print_unusual;
} config;

struct {
    double begin;
    double end;
    uint64_t messages;
    uint64_t queries;
    uint64_t multiqueries;	/* # of query msgs with >1 query */
    uint64_t zeroqueries;	/* # of query msgs with 0 queries */
    uint64_t recursion;		/* # of query msgs with RD set */
    uint64_t L4_errs;		/* # of msgs with unparseable transport hdr */
    uint64_t htrunc;		/* # of msgs lost b/c header was truncated */
    uint64_t qtrunc;		/* # of queries lost b/c msg was truncated */
    /*uint64_t bytes;*/
} interval_data;

#define FLOW_HASH_TABLE_SIZE		229	/* XXX */
#define QUERY_HASH_TABLE_SIZE		97843	/* XXX */

typedef struct querystats {
/* key */
    struct flowstats *flowstat;	/* flowstat to which this belongs */
    uint8_t opcode;
    uint16_t qtype;
    uint16_t qclass;

/* data */
    uint64_t queries;	    	/* # of occurances of this key (>= messages) */
    uint64_t bytes;		/* # of bytes for this query type */
    struct timespec first;
    struct timespec latest;

/* operational */
    struct querystats *next;	/* next querystats_t in flow's list */
} querystats_t;

typedef struct flowstats {
/* key */
    struct in_addr src;
    struct in_addr dst;
    uint8_t ip_proto;

/* data */
    uint64_t messages;		/* # of messages matching this key */
    uint64_t queries;		/* # of queries in messages matching this key */
    uint64_t recursion;		/* # of query msgs with RD set */
    uint64_t multiqueries;	/* # of query msgs with >1 query */
    uint64_t zeroqueries;	/* # of query msgs with 0 queries */
    uint64_t bytes;		/* # of bytes for this query type */
    uint64_t L4_errs;		/* # of msgs with unparseable transport hdr */
    uint64_t htrunc;		/* # of msgs lost b/c header was truncated */
    uint64_t qtrunc;		/* # of queries lost b/c msg was truncated */
    struct timespec first;
    struct timespec latest;

/* operational */
    struct querystats *querylist;	/* linked list of querystats_t's */
} flowstats_t;


static const char *qtype_label[T_ANY+1];
static const char *qclass_label[C_ANY+1];
static hash_tab *query_hash;
static hash_tab *flow_hash;
static int interrupted = 0, stopped = 0;

static void init_labels(void)
{
    /* from ftp://ftp.isi.edu/in-notes/iana/assignments/dns-parameters */
    qtype_label[1] = "A";
    qtype_label[2] = "NS";
    qtype_label[3] = "MD";
    qtype_label[4] = "MF";
    qtype_label[5] = "CNAME";
    qtype_label[6] = "SOA";
    qtype_label[7] = "MB";
    qtype_label[8] = "MG";
    qtype_label[9] = "MR";
    qtype_label[10] = "NULL";
    qtype_label[11] = "WKS";
    qtype_label[12] = "PTR";
    qtype_label[13] = "HINFO";
    qtype_label[14] = "MINFO";
    qtype_label[15] = "MX";
    qtype_label[16] = "TXT";
    qtype_label[17] = "RP";
    qtype_label[18] = "AFSDB";
    qtype_label[19] = "X25";
    qtype_label[20] = "ISDN";
    qtype_label[21] = "RT";
    qtype_label[22] = "NSAP";
    qtype_label[23] = "NSAP-PTR";
    qtype_label[24] = "SIG";
    qtype_label[25] = "KEY";
    qtype_label[26] = "PX";
    qtype_label[27] = "GPOS";
    qtype_label[28] = "AAAA";
    qtype_label[29] = "LOC";
    qtype_label[30] = "NXT";
    qtype_label[31] = "EID";
    qtype_label[32] = "NIMLOC";
    qtype_label[33] = "SRV";
    qtype_label[34] = "ATMA";
    qtype_label[35] = "NAPTR";
    qtype_label[36] = "KX";
    qtype_label[37] = "CERT";
    qtype_label[38] = "A6";
    qtype_label[39] = "DNAME";
    qtype_label[40] = "SINK";
    qtype_label[41] = "OPT";
    qtype_label[100] = "UINFO";
    qtype_label[101] = "UID";
    qtype_label[102] = "GID";
    qtype_label[103] = "UNSPEC";
    qtype_label[249] = "TKEY";
    qtype_label[250] = "TSIG";
    qtype_label[251] = "IXFR";
    qtype_label[252] = "AXFR";
    qtype_label[253] = "MAILB";
    qtype_label[254] = "MAILA";
    qtype_label[255] = "ANY";

    /* from ftp://ftp.isi.edu/in-notes/iana/assignments/dns-parameters */
    qclass_label[1] = "IN";
    qclass_label[3] = "CH";
    qclass_label[4] = "HS";
    qclass_label[254] = "NONE";
    qclass_label[255] = "ANY";
}

static const char *strqtype(uint16_t qtype)
{
    static char buf[8];
    if (qtype <= T_ANY && qtype_label[qtype])
	return qtype_label[qtype];
    sprintf(buf, "%d", qtype);
    return buf;
}

static const char *strqclass(uint16_t qclass)
{
    static char buf[8];
    if (qclass <= C_ANY && qclass_label[qclass])
	return qclass_label[qclass];
    sprintf(buf, "%d", qclass);
    return buf;
}


/* ---- hash table per flow fxns begin ---- */
/* -- per flow hash table fxns -- */

/* return 0 if the same - for use by the hashtable */
static int compare_flowstats(const void *entry1, const void *entry2)
{
    const flowstats_t *f1 = entry1;
    const flowstats_t *f2 = entry2;

    return (f1->ip_proto != f2->ip_proto ||
	    f1->src.s_addr != f2->src.s_addr ||
	    f1->dst.s_addr != f2->dst.s_addr);
}

/* make a hash of an entry - for use by the hashtable */
static unsigned long make_key_flowstats(const void *entry)
{
    const flowstats_t *f = entry;

    return (f->ip_proto << 16) + (f->src.s_addr * 59) + (f->dst.s_addr);
}

/* free mem of an entry - for use by the hashtable */
static void delete_flowstats(void *entry)
{
    if (!entry) return;
    /*free(entry);*/
}
/* -- end of flowstats_t hash table fxns -- */


/* ---- stuff that is per querystat hash table ---- */

/* return 0 if the same - for use by the hashtable */
static int compare_querystats(const void *entry1, const void *entry2)
{
    const querystats_t *q1 = entry1;
    const querystats_t *q2 = entry2;

    return (q1->flowstat != q2->flowstat ||
	    q1->opcode != q2->opcode ||
	    q1->qtype != q2->qtype ||
	    q1->qclass != q2->qclass);
}

/* make a hash of an entry - for use by the hashtable */
static unsigned long make_key_querystats(const void *entry)
{
    const querystats_t *q = entry;
    
    return ((unsigned)q->flowstat) +
	((q->opcode<<24) | (q->qclass<<16) | (q->qtype));
}

/* free mem of an entry - for use by the hashtable */
static void delete_querystats(void *entry)
{
    if (!entry) return;
    /*free(entry);*/
}
/* ---- hash table fxns end ---- */

/* NB: returns a pointer to static data */
static const char *hostname(const struct in_addr *addr)
{
    struct hostent *h;
    if (config.names &&
	(h = gethostbyaddr((const char*)addr, sizeof(struct in_addr), AF_INET)))
    {
	return h->h_name;
    } else {
	return inet_ntoa(*addr);
    }
}

static void count_flowstats(coral_iface_t *iface,
    coral_pkt_result_t *pkt_result)
{
    flowstats_t *flowrec;
    flowstats_t fkey;
    coral_pkt_buffer_t L3pdu;	/* network layer datagram (IP) */
    coral_pkt_buffer_t L4pdu;	/* transport layer packet (TCP/UDP) */
    coral_pkt_buffer_t dnspdu;	/* DNS message */
    querystats_t qkey;
    querystats_t *queryrec;
    HEADER *dnshdr;
    int i, offset, unusual = 0;
    uint16_t qdcount;
    struct ip *ip;

    if (coral_get_payload_by_layer(pkt_result->packet, &L3pdu, 3) < 0) {
	/*interval_data.L2_err++;*/
	return;
    }
    ip = (struct ip*)L3pdu.buf;
    if (coral_get_payload(&L3pdu, &L4pdu) < 0) {
	/*interval_data.L3_err++;*/
	return;
    }
    if (coral_get_payload(&L4pdu, &dnspdu) < 0) {
	/*flowrec->L4_errs++;*/
	return;
    }
    if (dnspdu.caplen < sizeof(HEADER)) {
	/*flowrec->htrunc++;*/
	return;
    }

    /*flowrec->bytes += ntohs(ip->ip_len);*/
    /*interval_data.ipbytes += ntohs(ip->ip_len);*/

    dnshdr = (HEADER*)dnspdu.buf;
    offset = sizeof(HEADER);

    if (dnshdr->qr)	/* response */
	return;

    fkey.src.s_addr = config.nosrc ? 0 : ip->ip_src.s_addr & config.ip_mask;
    fkey.dst.s_addr = config.nodst ? 0 : ip->ip_dst.s_addr & config.ip_mask;
    fkey.ip_proto = config.noproto ? 0 : ip->ip_p;

    if (!(flowrec = (flowstats_t*)find_hash_entry(flow_hash, &fkey))) {
	flowrec = (flowstats_t*)malloc(sizeof(flowstats_t));
	if (!flowrec) {
	    fprintf(stderr, "can't malloc flowstats_t.\n");
	    abort();
	}
	flowrec->src = fkey.src;
	flowrec->dst = fkey.dst;
	flowrec->ip_proto = fkey.ip_proto;

	flowrec->messages = 0;
	flowrec->queries = 0;
	flowrec->recursion = 0;
	flowrec->multiqueries = 0;
	flowrec->zeroqueries = 0;
	flowrec->bytes = 0;
	flowrec->L4_errs = 0;
	flowrec->htrunc= 0;
	flowrec->qtrunc= 0;
	flowrec->querylist = NULL;

	CORAL_TIMESTAMP_TO_TIMESPEC(iface, pkt_result->timestamp,
	    &flowrec->latest);
	flowrec->first = flowrec->latest;
        add_hash_entry(flow_hash, flowrec);
    } else {
	CORAL_TIMESTAMP_TO_TIMESPEC(iface, pkt_result->timestamp,
	    &flowrec->latest);
    }

    qdcount = ntohs(dnshdr->qdcount);
    if (qdcount != 1) {
	if (qdcount == 0) {
	    interval_data.zeroqueries++;
	    flowrec->zeroqueries++;
	} else {
	    interval_data.multiqueries++;
	    flowrec->multiqueries++;
	}
	unusual++;
	if (config.print_unusual)
	    fprintf(stderr, "message with %d queries\n", qdcount);
    }
    if (dnshdr->rd) {
	interval_data.recursion++;
	flowrec->recursion++;
    }

    interval_data.messages++;
    flowrec->messages++;

    interval_data.queries += qdcount;
    flowrec->queries += qdcount;

    if (config.notypes)
	goto end_count_flowstats;

    qkey.flowstat = flowrec;
    qkey.opcode = dnshdr->opcode;

    /* Header is followed by <qdcount> question sections.  A question section
     * contains an arbitrary length zero-terminated QNAME, a 2-octet QTYPE,
     * and a 2-octet QCLASS.  Note that the length of QNAME may be odd, so
     * QTYPE and QCLASS might not be short-aligned.
     */
    for (i = 0; i < qdcount; i++) {
	while (dnspdu.buf[offset++] && offset < dnspdu.caplen);
	if (offset + 2 * sizeof(short) > dnspdu.caplen)
	    break;
	memcpy(&qkey.qtype, dnspdu.buf + offset, 2);
	qkey.qtype = ntohs(qkey.qtype);
	offset += 2;
	memcpy(&qkey.qclass, dnspdu.buf + offset, 2);
	qkey.qclass = ntohs(qkey.qclass);
	offset += 2;

	if (config.print_unusual) {
	    if (qkey.qtype > T_ANY || !qtype_label[qkey.qtype]) {
		unusual++;
		fprintf(stderr, "query %d has qtype %d\n", i, qkey.qtype);
	    }
	    if (qkey.qclass > C_ANY || !qclass_label[qkey.qclass]) {
		unusual++;
		fprintf(stderr, "query %d has qclass %d\n", i, qkey.qclass);
	    }
	}

	/* Read the current value out of the database. */
	if (!(queryrec = (querystats_t*)find_hash_entry(query_hash, &qkey))) {
	    queryrec = (querystats_t*)malloc(sizeof(querystats_t));
	    if (!queryrec) {
		fprintf(stderr, "can't malloc querystats_t.\n");
		abort();
	    }

	    queryrec->flowstat = qkey.flowstat;
	    queryrec->opcode = qkey.opcode;
	    queryrec->qtype = qkey.qtype;
	    queryrec->qclass = qkey.qclass;

	    queryrec->queries = 0;
	    queryrec->bytes = 0;
	    queryrec->next = flowrec->querylist;
	    flowrec->querylist = queryrec;
	    queryrec->first = flowrec->latest;
	    add_hash_entry(query_hash, queryrec);
	}
	queryrec->queries++;
	/*queryrec->bytes += ntohs(ip->ip_len);*/
	queryrec->latest = flowrec->latest;
    }

end_count_flowstats:
    if (config.print_unusual && unusual) {
	coral_fmprint_pkt(stderr, iface, pkt_result->timestamp, 3, 5,
	    pkt_result->packet, NULL, NULL);
    }
}
/* ---- per flow stuff ends ---- */

static void init_hashes(void)
{
    query_hash = init_hash_table("# hashes entries for each query type",
			       compare_querystats, make_key_querystats,
			       delete_querystats, QUERY_HASH_TABLE_SIZE);
    flow_hash = init_hash_table("# hashes entries for each flow",
			       compare_flowstats, make_key_flowstats,
			       delete_flowstats, FLOW_HASH_TABLE_SIZE);
}

static void dump_hash_stats(void)
{
    dump_hashtab_stats(query_hash);
    dump_hashtab_stats(flow_hash);
}

static void dump_data(void)
{
    const flowstats_t *flowrec;
    double begin, end, intduration;
    querystats_t *queryrec;

    begin = interval_data.begin;
    end = interval_data.end;

    if (end && begin) {
	intduration = end - begin;
        printf("# begin trace interval at %f, duration %f\n",
	    begin, intduration);
	printf("# DNS messages: %" PRIu64 " (%f/s); "
	    "DNS queries: %" PRIu64 " (%f/s)\n\n", 
	    interval_data.messages, interval_data.messages / intduration,
	    interval_data.queries, interval_data.queries / intduration);

	printf("#"); 
	if (!config.nosrc)
	    printf(config.names ? "%-27s " : "%-15s ", "src");
	if (!config.nodst)
	    printf(config.names ? "%-27s " : "%-15s ", "dst");
	if (!config.noproto)
	    printf("%3s ", "pro");
	if (!config.notypes)
	    printf("%3s %-5s %-5s ", "op", "type", "class");
	printf("%7s %7s" /*" %15s"*/, "queries" /*, "bytes" */, "msgs");
	if (!config.no_rd)
	    printf(" %7s", "rd");
	printf(" %s\n", "notes");
    } else {
	fprintf(stdout, "\n# no data processed during interval\n");
    }

    init_hash_walk(flow_hash);

    while ((flowrec = next_hash_walk(flow_hash))) {
	printf(" "); 
	if (!config.nosrc)
	    printf(config.names ? "%-27s " : "%-15s ", hostname(&flowrec->src));
	if (!config.nodst)
	    printf(config.names ? "%-27s " : "%-15s ", hostname(&flowrec->dst));
	if (!config.noproto)
	    printf("%3d ", flowrec->ip_proto);
	if (!config.notypes)
	    printf("%3s %-5s %-5s ", "-", "-", "-");

	printf("%7" PRIu64 /*" %15" PRIu64*/ " %7" PRIu64,
	    flowrec->queries /*, queryrec->bytes */, flowrec->messages);
	if (!config.no_rd)
	    printf(" %7" PRIu64, flowrec->recursion);
	if (flowrec->L4_errs > 0)
	    printf(" L4err=%" PRIu64, flowrec->L4_errs);
	if (flowrec->htrunc > 0)
	    printf(" htrunc=%" PRIu64, flowrec->htrunc);
	if (flowrec->qtrunc > 0)
	    printf(" qtrunc=%" PRIu64, flowrec->qtrunc);
	if (flowrec->multiqueries > 0)
	    printf(" mq=%" PRIu64, flowrec->multiqueries);
	if (flowrec->zeroqueries > 0)
	    printf(" zq=%" PRIu64, flowrec->zeroqueries);
	if (!config.no_rd && flowrec->recursion &&
	    flowrec->recursion != flowrec->queries)
		printf(" rd!=q");

	printf("\n");

	/* flowrec->querylist is empty if config.notypes */
	for (queryrec=flowrec->querylist; queryrec; queryrec=queryrec->next) {
	    /*if (!queryrec->messages) continue;*/
	    printf(" "); 
	    if (!config.nosrc)
		printf(config.names ? "%-27s " : "%-15s ",
		    config.human ? "\"" : hostname(&flowrec->src));
	    if (!config.nodst)
		printf(config.names ? "%-27s " : "%-15s ",
		    config.human ? "\"" : hostname(&flowrec->dst));
	    if (!config.noproto) {
		if (config.human)
		    printf("%3s ", "\"");
		else
		    printf("%3d ", flowrec->ip_proto);
	    }

	    printf("%3d %-5s %-5s %7" PRIu64 /*" %15" PRIu64*/ /*" %7" PRIu64*/,
		queryrec->opcode,
		strqtype(queryrec->qtype), strqclass(queryrec->qclass),
		queryrec->queries
		/*, queryrec->bytes */ /*, queryrec->messages */);

	    printf("\n");
	}
    }

    printf("# end trace interval\n\n");
    fflush(stdout);
}

static void clear_data(void)
{
    /*interval_data.ipbytes = 0;*/
    /*interval_data.notIP = 0;*/
    interval_data.messages = 0;
    interval_data.queries = 0;
    interval_data.recursion = 0;
    interval_data.multiqueries = 0;
    interval_data.zeroqueries = 0;
    clear_hash_table(query_hash);
    clear_hash_table(flow_hash);
}

static void stop(int sig)
{
    interrupted++;
}

int main(int argc, char *argv[])
{
    int retval = 0;
    int len;
    coral_iface_t *iface;
    coral_pkt_result_t pkt_result;
    coral_interval_result_t interval_result;
    struct timeval interval = { 300, 0 };
    unsigned int duration;
    int opt;

    coral_set_api(CORAL_API_PKT);
    coral_set_duration(0);
    coral_set_interval(&interval);
    coral_set_iomode(0, CORAL_RX, 3*48, 0);
    coral_set_options(0, CORAL_OPT_PARTIAL_PKT | CORAL_OPT_NORMALIZE_TIME);
    config.ip_mask = 0xFFFFFFFFul;
    coral_add_pcap_filter("udp dst port 53");

    while ((opt = getopt(argc, argv, "C:p:asSDQPurnh")) != -1) {
	switch (opt) {
	case 'C':
	    if (coral_config_command(optarg) < 0)
		exit(-1);
	    break;
	case 'a':
	    config.names = 1;
	    break;
	case 'p':
	    len = atoi(optarg);
	    if (len < 8 || len > 32) {
		fprintf(stderr, "cidr prefix must be between 8 and 32.\n");
		exit(-1);
	    }
	    config.ip_mask = 0xFFFFFFFFul << (32 - len);
	    config.ip_mask = htonl(config.ip_mask);
	    break;
	case 'n':
	    config.nolabels = 1;
	    break;
	case 'S':
	    config.nosrc = 1;
	    break;
	case 'D':
	    config.nodst = 1;
	    break;
	case 'Q':
	    config.notypes = 1;
	    break;
	case 'P':
	    config.noproto = 1;
	    break;
	case 'r':
	    config.no_rd = 1;
	    break;
	case 'u':
	    config.print_unusual = 1;
	    break;
	case 'h':
	    config.human = 1;
	    break;
	case 's':
	    config.hashstats = 1;
	    break;
	default:
	    coral_usage(argv[0], "[-p<len>] [-anSDQhru] <source>...\n"
		"-p<len> aggregate hosts by CIDR prefix length <len> (default: 32)\n"
		"-a      resolve IP addresses to hostnames (requires -p32)\n"
		"-n      print DNS code numbers, not symbols\n"
		"-S      ignore IP source address\n"
		"-D      ignore IP destination address\n"
		"-Q      don't count by query opcode/class/type\n"
		"-h      print in more human-friendly format\n"
		"-r      do not count msgs with RD set\n"
		"-u      print contents of unusual msgs to stderr\n");
	    exit(-1);
	}
    }

    while (optind < argc) {
	if (!coral_new_source(argv[optind]))
	    exit(-1);
	optind++;
    }

    init_hashes();
    config.noproto = 1;	/* Only UDP is supported, so there's no point */ 
    if (!config.nolabels)
	init_labels();
    if (config.ip_mask != 0xFFFFFFFF)
	config.names = 0;

    if (coral_open_all() <= 0)
	exit(-1);

    if (coral_start_all() < 0)
	exit(-1);

    signal(SIGINT, stop);

    coral_get_interval(&interval);
    interval_data.begin = 0;
    interval_data.end = 0;

    duration = coral_get_duration();
    if (duration)
	coral_diag(2, ("collection duration max set to %d second(s)\n", 
		       duration));

    coral_read_pkt_init(NULL, NULL, &interval);

    printf("# dnsstat output version: 0.2\n\n");

    while (1) {
	if (interrupted) {
	    if (!stopped) {
		coral_stop_all();
		stopped = 1;
	    }
	    if (interrupted > 1)
		break;
	}
	iface = coral_read_pkt(&pkt_result, &interval_result);

	if (!iface) {
	    if (errno == EINTR || errno == EAGAIN) continue;
	    retval = errno;
	    break;
	}

	if (!pkt_result.packet && !interval_result.stats) {
	    /* beginning of interval */
	    interval_data.begin = timevaltodouble(&interval_result.begin);

	} else if (pkt_result.packet) {
	    /* got packet */
	    count_flowstats(iface, &pkt_result);

	} else if (!pkt_result.packet && interval_result.stats) {
	    /* end of interval */
	    interval_data.end = timevaltodouble(&interval_result.end);

	    dump_data();
	    clear_data();

	    /* reset the interval time, and other timers */
	    interval_data.begin = 0;
	    interval_data.end = 0;
	}
    }

    coral_stop_all();

    /* clean up stuff */
    if (config.hashstats) dump_hash_stats();
/*    clear_data();*/
    return retval;
}
