/* 
 * Copyright 1999, 2000, 2001 The Regents of the University of California 
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include "config.h"
#include "libcoral.h"

static const char RCSid[] = "$Id: coral_cksum.c,v 1.6.8.2 2001/11/21 01:33:46 kkeys Exp $";

/* semi-optimized internet checksum calculator.
 * Individual segments must be 16-bit-aligned with len < 32768.
 * Entire message must have length < 32768.
 */
uint32_t coral_in_cksum_add(uint32_t psum, const void *data, int len)
{
    register uint32_t sum = psum;
    register uint16_t *s = (uint16_t *) data;

#if 0	/* This would allow entire message to have length >= 32768. */
    sum = (sum & 0xFFFF) + (sum >> 16);
#endif

    /* take care of final odd byte */
    if (len % 2)
        sum += (uint16_t) ((unsigned char*)s)[len-1];

    len /= 2;

    /* unrolled x8 loop reduces loop overhead */
    while (len > 8) {
        sum += s[0]; sum += s[1]; sum += s[2]; sum += s[3];
        sum += s[4]; sum += s[5]; sum += s[6]; sum += s[7];
        len -= 8;
        s += 8;
    }

    while (len > 0) {
        sum += *s++;
        len--;
    }

    return sum;
}

uint16_t coral_in_cksum_result(uint32_t psum)
{
    register uint32_t sum = psum;

    sum = (sum & 0xFFFF) + (sum >> 16);
    sum += (sum >> 16);
    return ~sum;
}

/* TODO: coral_in_cksum_change(sum, len, olddata, newdata) */
