/* 
 * Copyright 1999, 2000, 2001 The Regents of the University of California 
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */

static const char RCSid[] = "$Id: coral_clock.c,v 1.48.2.3 2001/11/21 01:33:47 kkeys Exp $";

#include "config.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>

#include "libcoral.h"
#include "libcoral_priv.h"
#include "coral_clock.h"

/* Seconds in each tick of the hardware clock */
#define FORE_OC3_SEC_PER_TICK		40e-9
#define APPTEL_OC12_SEC_PER_TICK	25e-9

inline void coral_read_clock_order(const coral_iface_t *iface,
    const coral_timestamp_t *src, coral_timestamp_t *dst)
{
#if 0
    coral_internal_stats.clock_order++;
#endif
    iface->iface_type->clock_read_order(iface, src, dst);
}

inline double coral_read_clock_double(coral_iface_t *iface,
    const coral_timestamp_t *t)
{
#if 0
    coral_internal_stats.clock_double++;
#endif
    return iface->iface_type->clock_read_double(iface, t,
	coral_config.flags & CORAL_OPT_NORMALIZE_TIME);
}

void coral_read_clock_sec_nsec(coral_iface_t *iface,
    const coral_timestamp_t *t, time_t *sec, long *nsec)
{
    coral_clock_sec_nsec_inline(iface, t, sec, nsec);
}

/*
 * Clock-reading routines: these convert a timestamp to a standard form.
 * The generic sec_nsec function calls the double function, and the generic
 * double function calls the sec_nsec function.  Each interface type must
 * implement either the sec_nsec or the double function, and may use the
 * generic function for the other one.
 */

void coral_generic_clock_sec_nsec(const coral_iface_t *iface,
    const coral_timestamp_t *t, time_t *sec, long *nsec, int normalize)
{
    double time_calc = iface->iface_type->clock_read_double(iface, t,
	normalize);
    *sec = (long) time_calc;
    *nsec = (long) ((time_calc - *sec) * 1e9);
}

double coral_generic_clock_double(const coral_iface_t *iface,
    const coral_timestamp_t *t, int normalize)
{
    time_t sec;
    long nsec;
    iface->iface_type->clock_read_sec_nsec(iface, t, &sec, &nsec, normalize);
    return (double)sec + (double)nsec * 1e-9;
}

