/* 
 * Copyright 1999, 2000, 2001 The Regents of the University of California 
 * All Rights Reserved 
 * 
 * Permission to use, copy, modify and distribute any part of this
 * CoralReef software package for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby
 * granted, provided that the above copyright notice, this paragraph
 * and the following paragraphs appear in all copies.
 * 
 * Those desiring to incorporate this into commercial products or use
 * for commercial purposes should contact the Technology Transfer
 * Office, University of California, San Diego, 9500 Gilman Drive, La
 * Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
 * UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
 * OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
 * OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
 * ANY PATENT, TRADEMARK OR OTHER RIGHTS.
 * 
 * The CoralReef software package is developed by the CoralReef
 * development team at the University of California, San Diego under
 * the Cooperative Association for Internet Data Analysis (CAIDA)
 * Program. Support for this effort is provided by the CAIDA grant
 * NCR-9711092, DARPA NGI Contract N66001-98-2-8922, DARPA NMS Grant
 * N66001-01-1-8909, and by CAIDA members.
 * 
 * Report bugs and suggestions to coral-bugs@caida.org.
 */

/* High-level functions for pcap interfaces (devices or tracefiles).
 *
 *  $Id: coral_iface_pcap.c,v 1.3.2.3 2001/11/21 01:33:47 kkeys Exp $
 *
 */

#include "config.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>

#include "libcoral.h"
#include "libcoral_priv.h"
#include "coral_clock.h"

static const char RCSid[] = "$Id: coral_iface_pcap.c,v 1.3.2.3 2001/11/21 01:33:47 kkeys Exp $";


static inline void pcap_clock_order(const coral_iface_t *iface,
    const coral_timestamp_t *src, coral_timestamp_t *dst)
{
    if (iface->iface_info.time_is_le) {
	dst->i[0] = crl_letohl(src->i[0]);
	dst->i[1] = crl_letohl(src->i[1]);
    } else {
	dst->i[0] = crl_betohl(src->i[0]);
	dst->i[1] = crl_betohl(src->i[1]);
    }
}

static void inline pcap_clock_sec_nsec(const coral_iface_t *iface,
    const coral_timestamp_t *t, time_t *sec, long *nsec, int normalize)
{
    long usec;
    if (iface->iface_info.time_is_le) {
	*sec  = crl_letohl(t->i[0]);
	usec = crl_letohl(t->i[1]);
    } else {
	*sec  = crl_betohl(t->i[0]);
	usec = crl_betohl(t->i[1]);
    }
    *nsec = usec * 1000;

    /* pcap timestamps are already in unix time, so normalize is a no-op. */
}

void coral_pcap_update_pkt_stats(coral_iface_t *iface)
{
    struct pcap_stat stats;

    if (iface->src->type.coral_type == CORAL_TYPE_PCAP)
	return; /* stats are not available for files */
    if (iface->eof)
	return; /* stats were updated by coral_pcaplive_stop() */

    if (pcap_stats(iface->src->file, &stats) != 0) {
	coral_diag(0, ("%s: %s\n", iface->src->name,
	    pcap_geterr(iface->src->file)));
	return;
    }

    iface->pkt_stats.pkts_recv = stats.ps_recv - iface->u.pcap.stats.ps_recv;
    iface->pkt_stats.pkts_drop = stats.ps_drop - iface->u.pcap.stats.ps_drop;
    iface->u.pcap.stats = stats;
}

static void pcap_iface_init(coral_iface_t *iface)
{
    iface->time_is_normal = 1;
}


coral_iface_type_t coral_iface_type_pcap = {
    { -1, -1, -1, -1, -1, -1, -1, -1 },	/* pcap doesn't have ATM cells */
    "pcap",
    CORAL_API_PKT,
    pcap_iface_init,
    pcap_clock_order,
    coral_generic_clock_double,
    pcap_clock_sec_nsec,
    NULL /* correct_clock */,
    NULL /* consume */,
    NULL /* consume_pkt */,
    NULL /* prep_pkt */,
    coral_pcap_update_pkt_stats,
    NULL /* close */,
    { { CORAL_RX, 68 }, 0, CORAL_PROTO_UNKNOWN, CORAL_PROTO_UNKNOWN,
	NULL }			/* used by pcaplive only */
};

