#!/usr/local/bin/perl

require "utils.pl";

use File::Basename;

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# Begin main code section
#

$kDefaultArgKey = '_default';

$gTopDir = `pwd`; chomp $gTopDir;
$gTestScriptName = "TEST";
$gScriptTemplateName = "t.testscript.pl";

# Create a hash of array who's keys correspond
# to applications we want to test.
# (XXX) Remember, this is what needs to be modified
# (XXX) to make this scheme work... 
# (XXX) Wouldn't it be nice if I parse the 'usage' message
# (XXX) from each application to generate this table?
#
# (XXX) The command-lines below are just examples!  
# (XXX) Somebody who knows more about these apps should choose
# (XXX) better values...
#
# (XXX) One (of many) shameless hacks is this:  those apps that
# (XXX) take a -o option (i.e. that generate an output file) should
# (XXX) use OUTFILE as the file name (see crl_fail below).
#

%gTestArgHash = (
# (XXX) This default value is optional. If you use it,
# (XXX) then tests which are undefined in this hash will
# (XXX) be called with this (the default) command-line options.
#	$kDefaultArgKey => [
#		"\"-C'duration 0' -C'interval 0'\"",
#		"\"-C'duration 1' -C'interval 1'\"",
#	],

	crl_fail => [
		"\"-C \'duration 0\' -C\'interval 0\' -o OUTFILE\"",
		"\"-C\'duration 1\' -C\'interval 1\' -o OUTFILE\"", 
	],

	crl_trace => [
		"\"-C\'duration 0\' \"",
		"\"-C\'duration 60\'\"",
	],

	crl_info => [
		"\"-C\'duration 0\' -C\'interval 0\'\"",
		"\"-C\'duration 60\' -C\'interval 60\'\"",
	],

# (XXX) We should choose a variety of iomode options to excercise
# (XXX) the next app...
#	crl_print => [
#		"\"-C\'duration 0\' -C\'interval 0\'\"",
#		"\"-C\'duration 60\' -C\'interval 60\"",
#	],

# (XXX) Ditto for this one
#	crl_print_pkt => [
#		"\"-C\'duration 0\' -C\'interval 0\'\"",
#		"\"-C\'duration 60\' -C\'interval 60\'\"",
#	],

# (XXX) And this one, too...
#	netnet => [
#		"\"-C\'interval 180\'\"",
#	],

#	traffic2 => [
#		"\"-C\'duration 0\' -C\'interval 60\'\"",
#		"\"-C\'duration 60, -C\'interval 120\'\"",
#	],

#	crl_bycountry
#	crl_encode
#	crl_flowscnt
#	crl_hist
#	crl_hostmatrix
#	crl_ipaddr
#	crl_ipmatric
#	crl_llcsnap
#	crl_llcsnap2
#	crl_locality
#	crl_nonip
#	crl_portnummary
#	crl_sample
#	crl_timestamps
#	crl_toascii
#	crl_tofr
#	crl_tofr+
#	crl_tos
#	crl_totsh
#	crl_trace
#	crl_trains

);


# The scripts template should be in the 'top' directory.
# (XXX) This really should be a flagged command-line argunment, with
# (XXX) the default directory being "."...
#
$gScriptTemplatePath = "$gTopDir/$gScriptTemplateName";
if (! -f $gScriptTemplatePath)
{
	print "$gScriptTemplatePath\n";
	die "Error - couldn't establish an absolute path to the template!\n";
}

# First nuke old test scripts
RecursiveDescent($gTopDir, \&DeleteTestScript);

# Generate test scripts. Make sure to return to the starting directory.
RecursiveDescent($gTopDir, \&GenerateTestScript);

exit(0);

# End of main code section
#
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

sub DeleteTestScript
{
	if (0 == ListSubDirectories("."))
	{
		my @fileList = ListFiles(".");
		my $aFile;
		foreach $aFile (@fileList)
		{
			if (-f $aFile)
			{
				unlink $aFile
			 	 or die "Error - Could not delete $gTestScriptName";
			}
		}
	}
}

sub GenerateTestScript
{
	my $thisDir = `pwd`; chomp $thisDir;

	# Test scripts should exist in 'leaf' directories exclusively...
	# In other words, look for test scripts in directories that
	# don't have sub-directories.
	#
	if (0 == ListSubDirectories("."))
	{
		my $appName = basename($thisDir);
		# Look for a test script in this directory.
		#
		if (-f $gTestScriptName)
		{
			warn "$gTestScriptName already exists, leaving untouched\n";
		}
		else
		{

			if (defined $gTestArgHash{ $appName })
			{
				MakeAScript($thisDir, 
			 	 "$gTopDir/$gScriptTemplateName",
				 $gTestScriptName,
				 $appName,
				 $appName,
				 \%gTestArgHash);

				warn "created script for $appName\n";
			}
			elsif (defined $gTestArgHash{ $kDefaultArgKey })
			{
				warn "  warning- using default arg array for $appName test script\n";
				MakeAScript($thisDir,
				 "$gTopDir/$gScriptTemplateName",
				 $gTestScriptName, 
				 $appName,
				 $kDefaultArgKey,
				 \%gTestArgHash);

				warn "created script for $appName\n";
			}
			else
			{
				warn "  skipped $appName, no args defined...\n";
			}
		}
	}
}


sub MakeAScript
{
	my $thisDir = shift;
	my $templatePath = shift;
	my $scriptName = shift;
	my $appName = shift;
	my $hashKey = shift;
	my $hashRef = shift;

	# (XXX) Copy the template test script to this directory,
	# 'pasting' in the correct command-line argument array.
	#
	# (XXX) There's undoubtedly a clever unix command to do this,
	# but...
	#
	# Note that I'm not checking for the existance of the template
	# file here... That's because I'm assuming I've already done that.
	# 
	open SOURCEFILE, $templatePath
	 or die "Error - Unable to open $templatePath. $!\n";

	open SINKFILE, ">$scriptName"
	 or die "Error - Unable to open ${thisDir}/${scriptName}. $!\n";

	my $inLine;
	while ($inLine = <SOURCEFILE>)
	{
		if ($inLine =~ m/insert argument array here/)
		{
			print SINKFILE "\$gAppName = \'$appName\';", "\n";
			print SINKFILE '@gOptions =', "\n";
			print SINKFILE '(', "\n";
					
			my $line;
			foreach $line (@ {$hashRef->{$hashKey} })
			{
				print SINKFILE $line, ",\n";
			}

			print SINKFILE ');', "\n";
		}
		else
		{
			print SINKFILE $inLine;
		}
	}

	close SINKFILE;
	close SOURCEFILE;
	chmod 0777, $scriptName;
}
