#!/usr/local/bin/perl

require "utils.pl";

use File::Basename;
use Getopt::Std;

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# Begin main code section
#

# Globals
#
$gTestScriptName = "TEST";
$gBenchmarkName = "BENCH";


# Parse the command-line
#
if (! getopts('b')) {
    Usage();
}

if (@ARGV) {
    @gTraceList = @ARGV;
} else {
    print "You must specify one or more trace files on the command-line";
    Usage();
}

# Generate a timestamp value to use as a file suffix for the files
# generated by each internal test script.   I'll pass this value to 
# each local script, rather than have the local scripts generate one 
# themselves... This will make it trivial to tell which outputs belong
# to which test run.
#
$gTimeStamp = TimeStamp();

if ($opt_b) {
    $gTimeStamp = $gBenchmarkName;
}

$gTopDir = `pwd`;
chomp $gTopDir;


# Now recursively enter each subdirectory, and run any test scripts
# you might find there.
#
RecursiveDescent($gTopDir, \&RunTestScript);

#MakeBenchmarks();


exit(0);

# End of main code section
#
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


sub MakeBenchmarks
{
	$gTimeStamp = $gBenchmarkName;

	chdir $gTopDir;
	RecursiveDescent($gTopDir, \&RunTestScript);
}

sub RunTestScript
{
    my $thisDir = `pwd`;
    chomp $thisDir;

    # Test scripts should exist in 'leaf' directories exclusively...
    # In other words, look for test scripts in directories that
    # don't have sub-directories.
    #
    if (0 == ListSubDirectories(".")) {
	# Look for a test script in this directory.
	#
	if (-f $gTestScriptName) {
	    print STDERR "--- Running test in $thisDir ---\n";
	    my $command = "./$gTestScriptName $gTimeStamp @gTraceList";
	    print STDERR "$command\n";
	    my $returnVal = system $command;
	    $returnVal /= 256;

	    if (0 != $returnVal) {
		print STDERR "Error - script ${thisDir}/${gTestScriptName}",
			    " returned $returnVal\n";
	    } else {
		print STDERR "--- test complete ---\n";
	    }
	} else {
#		print STDERR "Error - missing test script: $thisDir\n";
	}
    }
}

sub Usage
{
	print "\n\n";
	print "$0 [trace file] {...[trace file]}\n";
	exit(1);
}
